package studentSystem;

import java.util.Objects;

public class Course  implements Comparable<Course>{ 
	
	String code;
	String title;
	Prof instructor;
	char term;
	int year;
		
	public Course(String code, String title, Prof instructor, int year, char term) {
		super();
		this.code = code;
		this.title = title;
		this.instructor = instructor;
		this.term = term;
		this.year = year;
	}
	
	public String getCode() {
		return code;
	}
	
	public void setCode(String code) {
		this.code = code;
	}
	
	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public Prof getInstructor() {
		return instructor;
	}


	public void setInstructor(Prof instructor) {
		this.instructor = instructor;
	}


	public char getTerm() {
		return term;
	}


	public void setTerm(char term) {
		this.term = term;
	}

	public int getYear() {
		return year;
	}

	public void setYear(int year) {
		this.year = year;
	}

	@Override
	public String toString() {
		//return "<"+this.codecode +" ", " + name + ", taught by:" + instructor.getName() + ", enrolment:" + enrolment
			//	+ ">";
		return String.format("<%s, %s, %s%c, taught by: %s>", this.code, this.title, this.year, this.term, this.instructor.getName());
	}

	

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((code == null) ? 0 : code.hashCode());
		result = prime * result + ((instructor == null) ? 0 : instructor.hashCode());
		result = prime * result + term;
		result = prime * result + year;
		return result;
	}
	
	/**
	 * This class defined the logical equality as follows:
	 * two Courses are equal if they have the same code, same instructor, and same year and term 
	 **/
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Course other = (Course) obj;
		if (code == null) {
			if (other.code != null)
				return false;
		} else if (!code.equals(other.code))
			return false;
		if (instructor == null) {
			if (other.instructor != null)
				return false;
		} else if (!instructor.equals(other.instructor))
			return false;
		if (term != other.term)
			return false;
		if (year != other.year)
			return false;
		return true;
	}
	
	/**
	 * This class defines the natural ordering of Course objects, as follow:
	 * When 'this' course is compared to the specified course, 
	 * 'this' course is "less than" the specified course if its year is earlier than that of the specified course,
	 * and is "greater than" the specified course if its year is later than that of the specified course.
	 * If the years are the same, then they are further compared based on the term, which is either 'W' or 'S' or 'F'.
	 * 'this' course is "less than" the specified course if its term is earlier than 
	 * that of the specified course, and is "greater than" the specified course if the term is later than that of the specified course.
	 * 
	 * Assume term is either 'W' or 'S' or 'F', and we define 'W' term is earlier than 'S', which is earlier than 'F'
	 * 
	 * If the term is also same, then they are further compared bases on the lexicographical ordering of the course code.
	 * 
	 * 'this' course is "less than" the specified course if its course code lexicographically precedes that of the other course, and is 'greater than' the
	 * specified course if its course code lexicographically follows that of the specified course.  
	 * E.g., EECS1710 lexicographically precedes EECS1720, which precedes EECS2030, which  precedes EECS2031 and so on.
	 * 
	 * The result is 0 is the course code (also) equals.
	 **/
	
	
	 /* @param other a course to compare
	 * @return a negative value if the courseCode lexicographically precedes that of the other, a positive number otherwise.
	 * if the course code is the same, then negative number if this course's enrolment is smaller than that of the other couse, and a positive number otherwise
	 */

	@Override
	public int compareTo(Course other) {
		
		if (this.year < other.year)
			return -1;
		else if (this.year > other.year)
			return 2;
		
		if (this.term > other.term)
			return -2;
		else if (this.term < other.term)
			return 2;
		
		return this.code.compareTo(other.code);
				 
	}

	
			
		
		
}

	


